using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

namespace Syntriax.Engine.Core;

public static class UniverseObjectExtensions
{
    public static T SetUniverseObject<T>(this T universeObject, string? name = "", IUniverseObject? parent = null) where T : IUniverseObject
    {
        if (!string.IsNullOrWhiteSpace(name))
            universeObject.Name = name;
        if (parent is not null)
            universeObject.SetParent(parent);
        return universeObject;
    }

    #region Universe Object Search
    public static T? GetUniverseObject<T>(this IEnumerable<IUniverseObject> universeObjects) where T : class
    {
        foreach (IUniverseObject universeObject in universeObjects)
            if (universeObject is T @object)
                return @object;

        return default;
    }

    public static bool TryGetUniverseObject<T>(this IEnumerable<IUniverseObject> universeObjects, [NotNullWhen(returnValue: true)] out T? universeObject) where T : class
    {
        universeObject = GetUniverseObject<T>(universeObjects);
        return universeObject is not null;
    }

    public static void GetUniverseObjects<T>(this IEnumerable<IUniverseObject> universeObjects, IList<T> foundUniverseObjects) where T : class
    {
        foundUniverseObjects.Clear();

        foreach (IUniverseObject universeObject in universeObjects)
            if (universeObject is T @object)
                foundUniverseObjects.Add(@object);
    }
    #endregion

    #region Behaviour Search
    public static T? FindBehaviour<T>(this IEnumerable<IUniverseObject> universeObjects) where T : class
    {
        foreach (IUniverseObject universeObject in universeObjects)
            if (universeObject.BehaviourController.GetBehaviour<T>() is T behaviour)
                return behaviour;

        return default;
    }

    public static bool TryFindBehaviour<T>(this IEnumerable<IUniverseObject> universeObjects, [NotNullWhen(returnValue: true)] out T? behaviour) where T : class
    {
        behaviour = FindBehaviour<T>(universeObjects);
        return behaviour is not null;
    }

    public static void FindBehaviours<T>(this IEnumerable<IUniverseObject> universeObjects, IList<T> behaviours) where T : class
    {
        behaviours.Clear();
        List<T> cache = [];

        foreach (IUniverseObject universeObject in universeObjects)
        {
            universeObject.BehaviourController.GetBehaviours(cache);
            foreach (T behaviour in cache)
                behaviours.Add(behaviour);
        }
    }
    #endregion
}
